@@ -18,6 +18,7 @@
 #' @param chunk_size      Number of function calls to chunk together
 #'                        defaults to 100 chunks per worker or max. 10 kb per chunk
 #' @param timeout         Maximum time in seconds to wait for worker (default: Inf)
+#' @param max_calls_worker  Maxmimum number of function calls that will be sent to one worker
 #' @return                A list of whatever `fun` returned
 #' @export
 #'
@@ -36,7 +37,7 @@
 Q = function(fun, ..., const=list(), export=list(), seed=128965,
         memory=NULL, template=list(), n_jobs=NULL, job_size=NULL,
         split_array_by=-1, rettype="list", fail_on_error=TRUE, workers=NULL,
-        log_worker=FALSE, chunk_size=NA, timeout=Inf) {
+        log_worker=FALSE, chunk_size=NA, timeout=Inf, max_calls_worker=Inf) {
 
     split_arrays = function(x) {
         if (is.array(x))
@@ -61,5 +62,6 @@ Q = function(fun, ..., const=list(), export=list(), seed=128965,
            workers = workers,
            log_worker = log_worker,
            chunk_size = chunk_size,
-           timeout = timeout)
+           timeout = timeout,
+           max_calls_worker = max_calls_worker)
 }
