@@ -17,10 +17,11 @@
 #' @param chunk_size     Number of function calls to chunk together
 #'                       defaults to 100 chunks per worker or max. 500 kb per chunk
 #' @param timeout         Maximum time in seconds to wait for worker (default: Inf)
+#' @param max_calls_worker  Maxmimum number of function calls that will be sent to one worker
 #' @return               A list of whatever `fun` returned
 #' @keywords  internal
 master = function(qsys, iter, rettype="list", fail_on_error=TRUE,
-                  chunk_size=NA, timeout=Inf) {
+                  chunk_size=NA, timeout=Inf, max_calls_worker=Inf) {
     # prepare empty variables for managing results
     n_calls = nrow(iter)
     job_result = rep(vec_lookup[[rettype]], n_calls)
@@ -30,11 +31,14 @@ master = function(qsys, iter, rettype="list", fail_on_error=TRUE,
     n_errors = 0
     n_warnings = 0
     shutdown = FALSE
+    kill_workers = FALSE
 
     on.exit(qsys$finalize())
 
     message("Running ", format(n_calls, big.mark=",", scientific=FALSE),
-            " calculations (", chunk_size, " calls/chunk) ...")
+            " calculations (", qsys$data_num, " objs/",
+            format(qsys$data_size, big.mark=",", units="Mb"),
+            " common; ", chunk_size, " calls/chunk) ...")
     pb = progress::progress_bar$new(total = n_calls,
             format = "[:bar] :percent (:wup/:wtot wrk) eta: :eta")
     pb$tick(0)
@@ -42,6 +46,14 @@ master = function(qsys, iter, rettype="list", fail_on_error=TRUE,
     # main event loop
     while((!shutdown && submit_index[1] <= n_calls) || jobs_running > 0) {
         msg = qsys$receive_data(timeout=timeout)
+        if (is.null(msg)) { # timeout reached
+            if (shutdown) {
+                kill_workers = TRUE
+                break
+            } else
+                stop("Socket timeout reached, likely due to a worker crash")
+        }
+
         pb$tick(length(msg$result),
                 tokens=list(wtot=qsys$workers, wup=qsys$workers_running))
 
@@ -53,17 +65,24 @@ master = function(qsys, iter, rettype="list", fail_on_error=TRUE,
 
             n_warnings = n_warnings + length(msg$warnings)
             n_errors = n_errors + length(msg$errors)
-            if (n_errors > 0 && fail_on_error == TRUE)
+            if (n_errors > 0 && fail_on_error == TRUE) {
                 shutdown = TRUE
+                timeout = getOption("clustermq.error.timeout", min(timeout, 30))
+            }
             new_msgs = c(msg$errors, msg$warnings)
-            if (length(new_msgs > 0) && length(cond_msgs) < 50)
+            if (length(new_msgs) > 0 && length(cond_msgs) < 50)
                 cond_msgs = c(cond_msgs, new_msgs[order(names(new_msgs))])
         }
 
-        if (!shutdown && msg$token != qsys$data_token) {
+        if (shutdown || (!is.null(msg$n_calls) && msg$n_calls >= max_calls_worker)) {
+            qsys$send_shutdown_worker()
+            next
+        }
+
+        if (msg$token != qsys$data_token) {
             qsys$send_common_data()
 
-        } else if (!shutdown && submit_index[1] <= n_calls) {
+        } else if (submit_index[1] <= n_calls) {
             # if we have work, send it to the worker
             submit_index = submit_index[submit_index <= n_calls]
             qsys$send_job_data(chunk = chunk(iter, submit_index))
@@ -77,14 +96,14 @@ master = function(qsys, iter, rettype="list", fail_on_error=TRUE,
                 submit_index = submit_index[1:chunk_size]
             }
 
-        } else if (!shutdown && qsys$reusable) {
+        } else if (qsys$reusable) {
             qsys$send_wait()
-
-        } else # or else shut it down
+        } else { # or else shut it down
             qsys$send_shutdown_worker()
+        }
     }
 
-    if (qsys$reusable || qsys$cleanup())
+    if (!kill_workers && (qsys$reusable || qsys$cleanup()))
         on.exit(NULL)
 
     summarize_result(job_result, n_errors, n_warnings, cond_msgs,
